/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.thumbs;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

/**
 * Use ThumbFactory like this
 * 
 * <pre>
 * BufferedImage image = ImageIO.read(in);                
 * image = ThumbFactory.getThumbnail(image, width, height);
 * ImageIO.write(image, "png", response.getOutputStream());
 * </pre>
 * 
 * @author tiwe
 * @version $Id$
 */
public class ThumbFactory {

    public static BufferedImage getThumbnail(BufferedImage image,
            int maxThumbWidth, int maxThumbHeight) {         
        boolean cont = true;
        while(cont){            
            double scale = scaleToFit(image.getWidth(), image.getHeight(),
                    maxThumbWidth, maxThumbHeight);    
            int width, height;
            if (scale < 0.5){
                scale = 0.5;
                width = (int)(image.getWidth() * scale);
                height = (int)(image.getHeight() * scale);
            }else{
                cont = false;
                width = maxThumbWidth;
                height = maxThumbHeight;
            }
            AffineTransform tx = new AffineTransform();            
            tx.scale(scale, scale);
            
            // create thumbnail
            BufferedImage thumbnail = new BufferedImage(
                    width,
                    height,
                    BufferedImage.TYPE_INT_RGB);
            Graphics2D g2d = thumbnail.createGraphics();
            g2d.setBackground(new Color(255,255,255));
            g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);                
            g2d.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
            g2d.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY);
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            g2d.setRenderingHint(RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_ENABLE);
            g2d.clearRect(0, 0, width, height);
            g2d.drawImage(image, tx, null);
            g2d.dispose();            

            image = thumbnail;
        }
        
        return image;
    }

    private static double scaleToFit(double w1, double h1, double w2, double h2) {
        double scale = 1.0D;
        if (w1 > h1) {
            if (w1 > w2) scale = w2 / w1;
            h1 *= scale;
            if (h1 > h2) scale *= h2 / h1;
        } else {
            if (h1 > h2) scale = h2 / h1;
            w1 *= scale;
            if (w1 > w2) scale *= w2 / w1;
        }
        return scale;
    }

}
